PRO readPDS_SPICAV_UV
;-----------------------------------------------------------------------------
; NAME: readPDS_SPICAV_UV
;
; PURPOSE: To read a SPICAV UV data file.
;          This routine calls the readPDS set of IDL procedures developed at
;          the SBN (http://pdssbn.astro.umd.edu/nodehtml/software.shtml)
;          Version 01 of readSBN_UV works with readPDS version 4.3 Package
;          Nevertheless note that a small modification has been performed
;          in the pointpds.pro routine on lines 150 and 175.
;          (bug with call to the stregex routine under windows)
;
; CALLING SEQUENCE: readSBN_UV
;
;
; PROCEDURES USED:
;     Functions: READPDS
;
; MODIFICATION HISTORY:
;     Written by Aurelie Reberac [May 27, 2008]
;
;-----------------------------------------------------------------------------

   ;
   ; The SBN routines read data array and store it in a IDL structure
   ; readpds.pro is the 'top level' program
   ;
   ; The following message will be displayed:
   ; Now reading RECORD_ARRAY ARRAY/COLLECTION object
   ; ** Structure <4396440>, 2 tags, length=2532866, data length=2532866, refs=1:
   ; OBJECTS         INT              1
   ; RECORD_ARRAY    STRUCT    -> <Anonymous> Array[582]
   ;
   ;
   ; To access the RECORD_ARRAY and the different tags, type :
   ;
   ;  > help,/struct,data.record_array
   ; ** Structure <183d288>, 1 tags, length=4352, data length=4352, refs=2:
   ; ONE_SPICAV_UV_RECORD   STRUCT    -> <Anonymous> Array[1]
   ;
   ;  > help,/struct,data.record_array.one_spicav_uv_record
   ; ** Structure <1873db0>, 3 tags, length=4352, data length=4352, refs=2:
   ; HEADER_ARRAY    INT       Array[128]
   ; DATA_ARRAY      INT       Array[408,5]
   ; SPARE_ARRAY     INT       Array[8]
   ;
   ;  > help,/struct,data.record_array.one_spicav_uv_record.header_array
   ;  <Expression>    INT       = Array[128, 582]

   datapath = 'E:\VEXSPI_1001\SOFTWARE\SPICAV_READPDS43\DATA'

   ;select a data label file *.LBL
   fn = dialog_pickfile(PATH=datapath)
   data = readpds(fn)

   ;The header_array contains detector parameters values and time of observation
   ;example to access the header_array of the first and the last record
   first_header_array = data.record_array[0].one_spicav_uv_record.header_array
   nb_records = N_ELEMENTS(data.record_array)
   last_header_array = data.record_array[nb_records-1].one_spicav_uv_record.header_array

	code_op = first_header_array[40] ;operating code (100:full CCD - 101/102:5 bands)
	binning = first_header_array[46] ;binning (number of ccd lines analogically summed, 0 = progressive binning)
	nlig = first_header_array[45]
	ncol = first_header_array[44]

   print,'TIME_EXPOSURE',first_header_array[41] ; exposure time
   print,'CODE_OP',code_op
   print,'Y0',first_header_array[43] ;first line of CCD read
   print,'BIN',binning
   print,'FIRST TIME',first_header_array[60:66] ; UTC time of first measurement
   print,'LAST TIME',last_header_array[60:66]   ; UTC time of last measurement

   ;The data_array contains 5 CCD "bands" of 408 pixels, corresponding to one
   ;measurement recorded at one time t

   ;example to access the data_array of the first record
   first_data_array = data.record_array[0].one_spicav_uv_record.data_array

   ;In the ALIGN operating mode (code op = 100), a complete image of the CDD
   ;can be obtained during the observation by moving the position of the first
   ;band (=one row) read at each measurement. The initial position (at time t0)
   ;of the first band is Y0. At time T0+1,the position of the first band becomes Y0+4,
   ;at time T0+2, Y0+8, ...at time T0+p  Y0+p*4.
   ;Depending on the duration of the observation, several complete images can be obtained.

   ;In the BINNING (code_op=101) or PROGRESSIVE BINNING (code_op=102)
   ;operating mode, each band is a binning of BIN rows, with an identical
   ;BIN (= 2, 4, 8, 16 OR 32) for each band in the BINNING mode and a progressive
   ;binning, (band1=2, band2=4,band3=8,band4=16,band5=32)), in the PROGRESSIVE BINNING mode.
   ;In these modes,the CCD position of the bands read is fixed.
   ;At each measurement, the position of the first band in physical pixel is Y0,
   ;and the position of the second band is  Y0 + BIN, etc..


   ;In the BINNING or PROGRESSIVE BINNING mode, a temporal image can be obtained
   ;for each of the five bands.
   tempoimage_band1 = data.record_array.one_spicav_uv_record.data_array[*,0]
   tempoimage_band2 = data.record_array.one_spicav_uv_record.data_array[*,1]
   tempoimage_band3 = data.record_array.one_spicav_uv_record.data_array[*,2]
   tempoimage_band4 = data.record_array.one_spicav_uv_record.data_array[*,3]
   tempoimage_band5 = data.record_array.one_spicav_uv_record.data_array[*,4]

   ;In the ALIGN operating mode, complete image of the CCD can be obtained (290 rows of 408 pixels).
   ;Note that the increment of the position Y0 of the CCD row read is 4 at each new measurement.
   ;Therefore,the fifth "band" of one measurement corresponds with the first "band" of the next measurement.
   ;To build the image of the CCD, a mean of these 2 bands can be done.
    nb_CCDimage = nb_records/(290/4)
    nb_CCDimage = nb_CCDimage + 2 ; first and last image can be incomplete


	CCDimages = intarr(ncol,290,nb_CCDimage)
	yoff = intarr(nb_records)
    allData = intarr(ncol,nlig*nb_records)

    FOR ij = 0, nb_records-1 DO BEGIN
		 yoff(ij) = data.record_array[ij].one_spicav_uv_record.header_array[43]
         yr = nlig*ij
         allData(0:ncol-1,yr:yr+nlig-1) = data.record_array[ij].one_spicav_uv_record.data_array


	ENDFOR


    nim = 0
    ypre = 0

    for i = 0, nb_records-1 do begin

       bande = alldata(*,5*i:5*i+4)
       ybande = yoff(i) + indgen(5)

       if ypre gt ybande(0) then nim = nim + 1

       for j = 0,4 do begin
        if max(CCDimages(*,ybande(j),nim)) eq 0 then $
           CCDimages(*,ybande(j),nim) = bande(*,j) else $
           CCDimages(*,ybande(j),nim) = (CCDimages(*,ybande(j),nim) + bande(*,j))/2.
       endfor

       ypre = ybande(0)

    endfor

   ;plots
   LOADCT,39
   TVLCT,FSC_COLOR('Black',/TRIPLE),0
   TVLCT,FSC_COLOR('white',/TRIPLE),1
   TVLCT,FSC_COLOR('Red',/TRIPLE),2
   TVLCT,FSC_COLOR('Green',/TRIPLE),3
   TVLCT,FSC_COLOR('Blue',/TRIPLE),4
   TVLCT,FSC_COLOR('Yellow',/TRIPLE),5
   TVLCT,FSC_COLOR('magenta',/TRIPLE),6
   TVLCT,FSC_COLOR('orange',/TRIPLE),7

   !p.background = 1
   colorset, RETAIN=2, QUIET=QUIET

   window,0
   !p.multi = 0
   pixel=indgen(408)
   plot,pixel,first_data_array[*,0],xrange=[0,407],xstyle=1,color=0 ; plot the first band
   oplot,pixel,first_data_array[*,1],color=2 ; plot the second band
   oplot,pixel,first_data_array[*,2],color=3 ; plot the third band
   oplot,pixel,first_data_array[*,3],color=4 ; plot the fourth band
   oplot,pixel,first_data_array[*,4],color=6 ; plot the fifth band



   window,1

	if (code_op eq 101 or code_op eq 102 or (code_op eq 100 and binning eq 1)) then begin
	   !p.multi =[0,5,1]
	   contour,tempoimage_band1,/fill,nlevels=30,xrange=[0,407],xstyle=1,title='band1',color=0,xtitle='pixel'
	   contour,tempoimage_band2,/fill,nlevels=30,xrange=[0,407],xstyle=1,title='band2',color=0,xtitle='pixel'
	   contour,tempoimage_band3,/fill,nlevels=30,xrange=[0,407],xstyle=1,title='band3',color=0,xtitle='pixel'
	   contour,tempoimage_band4,/fill,nlevels=30,xrange=[0,407],xstyle=1,title='band4',color=0,xtitle='pixel'
	   contour,tempoimage_band5,/fill,nlevels=30,xrange=[0,407],xstyle=1,title='band5',color=0,xtitle='pixel'
	endif else begin
	   !p.multi =[0,3,2]
	   contour,CCDimages(*,*,0),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   if (nb_CCDimage gt 1) then begin
	   	if (max(CCDimages(*,*,1) ne 0)) then $
	   		contour,CCDimages(*,*,1),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	   if (nb_CCDimage gt 2)then begin
	   	if (max(CCDimages(*,*,2) ne 0)) then $
	   		contour,CCDimages(*,*,2),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	   if (nb_CCDimage gt 3)then begin
	   	if (max(CCDimages(*,*,3) ne 0)) then $
	   		contour,CCDimages(*,*,3),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	   if (nb_CCDimage gt 4)then begin
	   	if (max(CCDimages(*,*,4) ne 0)) then $
	   		contour,CCDimages(*,*,4),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	   if (nb_CCDimage gt 5)then begin
	   	if (max(CCDimages(*,*,5) ne 0)) then $
	   		contour,CCDimages(*,*,5),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	   if (nb_CCDimage gt 6)then begin
	   	if (max(CCDimages(*,*,6) ne 0)) then $
	   		contour,CCDimages(*,*,6),/fill,nlevels=30,xrange=[0,407],xstyle=1,color=0
	   endif

	endelse


END
